@@ -21,6 +21,7 @@ using namespace std;
 // Global state. The file interpreter.hpp also has some global state
 // in its 'th_params' variable
 global_state gs;
+long global_max_threads=DEFAULT_MAX_THREADS;
 
 
 /* Do the worker job for a certain thread */
@@ -55,9 +56,14 @@ void *th_worker(void *tidptr)
         pthread_mutex_lock(&gs.count_threads_mutex);
         if (gs.count_threads < gs.nthreads) {
             gs.count_threads++;
-            pthread_cond_wait(&gs.count_threads_cv, &gs.count_threads_mutex);
+            /* Beware of spurious wakeups. See issue pydata/numexpr#306. */
+            do {
+                pthread_cond_wait(&gs.count_threads_cv,
+                                  &gs.count_threads_mutex);
+            } while (!gs.barrier_passed);
         }
         else {
+            gs.barrier_passed = 1;
             pthread_cond_broadcast(&gs.count_threads_cv);
         }
         pthread_mutex_unlock(&gs.count_threads_mutex);
@@ -163,9 +169,13 @@ void *th_worker(void *tidptr)
         pthread_mutex_lock(&gs.count_threads_mutex);
         if (gs.count_threads > 0) {
             gs.count_threads--;
-            pthread_cond_wait(&gs.count_threads_cv, &gs.count_threads_mutex);
+            do {
+                pthread_cond_wait(&gs.count_threads_cv,
+                                  &gs.count_threads_mutex);
+            } while (gs.barrier_passed);
         }
         else {
+            gs.barrier_passed = 0;
             pthread_cond_broadcast(&gs.count_threads_cv);
         }
         pthread_mutex_unlock(&gs.count_threads_mutex);
@@ -185,6 +195,11 @@ int init_threads(void)
 {
     int tid, rc;
 
+    if ( !(gs.nthreads > 1 && (!gs.init_threads_done || gs.pid != getpid())) ) {
+        /* Thread pool must always be initialized once and once only. */
+        return(0);
+    }
+
     /* Initialize mutex and condition variable objects */
     pthread_mutex_init(&gs.count_mutex, NULL);
     pthread_mutex_init(&gs.parallel_mutex, NULL);
@@ -193,6 +208,7 @@ int init_threads(void)
     pthread_mutex_init(&gs.count_threads_mutex, NULL);
     pthread_cond_init(&gs.count_threads_cv, NULL);
     gs.count_threads = 0;      /* Reset threads counter */
+    gs.barrier_passed = 0;
 
     /* Finally, create the threads */
     for (tid = 0; tid < gs.nthreads; tid++) {
@@ -220,10 +236,16 @@ int numexpr_set_nthreads(int nthreads_new)
     int t, rc;
     void *status;
 
-    if (nthreads_new > MAX_THREADS) {
+    // if (nthreads_new > MAX_THREADS) {
+    //     fprintf(stderr,
+    //             "Error.  nthreads cannot be larger than MAX_THREADS (%d)",
+    //             MAX_THREADS);
+    //     return -1;
+    // }
+    if (nthreads_new > global_max_threads) {
         fprintf(stderr,
-                "Error.  nthreads cannot be larger than MAX_THREADS (%d)",
-                MAX_THREADS);
+                "Error.  nthreads cannot be larger than environment variable \"NUMEXPR_MAX_THREADS\" (%ld)",
+                global_max_threads);
         return -1;
     }
     else if (nthreads_new <= 0) {
@@ -240,9 +262,13 @@ int numexpr_set_nthreads(int nthreads_new)
         pthread_mutex_lock(&gs.count_threads_mutex);
         if (gs.count_threads < gs.nthreads) {
             gs.count_threads++;
-            pthread_cond_wait(&gs.count_threads_cv, &gs.count_threads_mutex);
+            do {
+                pthread_cond_wait(&gs.count_threads_cv,
+                                  &gs.count_threads_mutex);
+            } while (!gs.barrier_passed);
         }
         else {
+            gs.barrier_passed = 1;
             pthread_cond_broadcast(&gs.count_threads_cv);
         }
         pthread_mutex_unlock(&gs.count_threads_mutex);
@@ -264,9 +290,7 @@ int numexpr_set_nthreads(int nthreads_new)
 
     /* Launch a new pool of threads (if necessary) */
     gs.nthreads = nthreads_new;
-    if (gs.nthreads > 1 && (!gs.init_threads_done || gs.pid != getpid())) {
-        init_threads();
-    }
+    init_threads();
 
     return nthreads_old;
 }
@@ -384,6 +408,22 @@ initinterpreter()
 {
     PyObject *m, *d;
 
+
+    char *max_thread_str = getenv("NUMEXPR_MAX_THREADS");
+    char *end;
+    if (max_thread_str != NULL) {
+        global_max_threads = strtol(max_thread_str, &end, 10);
+    }
+
+
+    th_params.memsteps    = (npy_intp**)calloc(sizeof(npy_intp*), global_max_threads);
+    th_params.iter        = (NpyIter**)calloc(sizeof(NpyIter*),   global_max_threads);
+    th_params.reduce_iter = (NpyIter**)calloc(sizeof(NpyIter*),   global_max_threads);
+    gs.threads            = (pthread_t*)calloc(sizeof(pthread_t), global_max_threads);
+    gs.tids               = (int*)calloc(sizeof(int),             global_max_threads);
+    // TODO: for Py3, deallocate: https://docs.python.org/3/c-api/module.html#c.PyModuleDef.m_free
+    // For Python 2.7, people have to exit the process to reclaim the memory.
+
     if (PyType_Ready(&NumExprType) < 0)
         INITERROR;
 
@@ -437,6 +477,8 @@ initinterpreter()
     if (PyModule_AddObject(m, "allaxes", PyLong_FromLong(255)) < 0) INITERROR;
     if (PyModule_AddObject(m, "maxdims", PyLong_FromLong(NPY_MAXDIMS)) < 0) INITERROR;
 
+    if(PyModule_AddIntConstant(m, "MAX_THREADS", global_max_threads) < 0) INITERROR;
+
 #if PY_MAJOR_VERSION >= 3
     return m;
 #endif
